// DaysEaseDoc.cpp : implementation of the CDaysEaseDoc class
//
/*
	DaysEase - PC Magazine file download utility
	Copyright  1998-1999 Ziff-Davis, Inc.  All rights reserved.
	First published in PC Magazine, US Edition
	December 1, 1998.
	Written by Richard A. Ellingson
*/
#include "stdafx.h"
#include "DaysEase.h"

#include "DaysEaseDoc.h"
#include "MainFrm.h"
#include "DaysEaseView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
extern CDaysEaseApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CDaysEaseDoc

IMPLEMENT_DYNCREATE(CDaysEaseDoc, CDocument)

BEGIN_MESSAGE_MAP(CDaysEaseDoc, CDocument)
	//{{AFX_MSG_MAP(CDaysEaseDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseDoc construction/destruction

CDaysEaseDoc::CDaysEaseDoc()
{
	CFont Date, Notes, Title, Weekday;

	// We only have one Document, so save the pointer.
	theApp.Set_Doc_Pointer(this);

	// Set out new document flag.
	fNewDoc = TRUE;

	// We need a LOGFONT structure for each of our four fonts.
	// Rather than fill in each item of each structure,
	// we'll let the CreatePointFont() function do it.

	// Create the Date font LOGFONT structure.
	Date.CreatePointFont(40, theApp.csDateFont, NULL);
	Date.GetLogFont(&lfDate);
	// We only want ANSI characters.
	lfDate.lfCharSet = ANSI_CHARSET;
	// Initialize the weight and italic on/off from our
	// settings retrieved from DaysEase.ini when we launched.
	lfDate.lfWeight = theApp.DateWeight;
	lfDate.lfItalic = theApp.DateItalic;

	// Create the Day Notes font LOGFONT structure.
	Notes.CreatePointFont(40, theApp.csNotesFont, NULL);
	Notes.GetLogFont(&lfNotes);
	lfNotes.lfCharSet = ANSI_CHARSET;
	lfNotes.lfWeight = theApp.NotesWeight;
	lfNotes.lfItalic = theApp.NotesItalic;

	// Create the Title font LOGFONT structure.
	Title.CreatePointFont(40, theApp.csTitleFont, NULL);
	Title.GetLogFont(&lfTitle);
	lfTitle.lfCharSet = ANSI_CHARSET;
	lfTitle.lfWeight = theApp.TitleWeight;
	lfTitle.lfItalic = theApp.TitleItalic;

	// Create the Week Day font LOGFONT structure.
	Weekday.CreatePointFont(40, theApp.csWkdayFont, NULL);
	Weekday.GetLogFont(&lfWeekday);
	lfWeekday.lfCharSet = ANSI_CHARSET;
	lfWeekday.lfWeight = theApp.WkdayWeight;
	lfWeekday.lfItalic = theApp.WkdayItalic;
	
	// Since we only need the LOGFONT structures
	// we'll delete the fonts themselves and recreate them
	// from the LOGFONT structures as needed.
	// Note that we will always delete the fonts when we're done
	// with them to prevent memory leaks.
	Date.DeleteObject();
	Notes.DeleteObject();
	Title.DeleteObject();
	Weekday.DeleteObject();
}

CDaysEaseDoc::~CDaysEaseDoc()
{
}

BOOL CDaysEaseDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	if(!fNewDoc)
		return TRUE;
		
// Reset the following only when the new document flag is set.
	for(int x=0; x<7; x++)
		for(int y=0; y<6; y++)
			m_grid[x][y].Empty();
	theApp.Empty_Event_Array();
	csImageName = "no image loaded";
	ratio_top = -1;
	fGraphic = 0;
	nMonth = 0;
	fMonYr = 0;
	fTransparent = 0;
	PrintColor = theApp.PrintColor;
	PaperSize = theApp.PaperSize;
	Orientation = theApp.Orientation;
	PrinterName = theApp.PrinterName;
	cstitle.Empty();
	nTitleLarge = 0;

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CDaysEaseDoc serialization

void CDaysEaseDoc::Serialize(CArchive& ar)
{
	CString EventFile("");
	CMainFrame* pMain = theApp.Main_Pointer();

	if (ar.IsStoring())
	{
		for (int x=0; x<7; x++)
			for (int y=0; y<6; y++)
				ar << m_grid[x][y];

		if(!fMonYr) // If this is a Month calendar, save the Event File name.
			pMain->m_EventsDlg.m_EventIniBox.GetWindowText(EventFile);

		ar << EventFile;
		ar << nYear;
		ar << nMonth;
		ar << fMonYr;
		ar << fTransparent;
		ar << cstitle;
		ar << ratio_left;
		ar << ratio_top;
		ar << ratio_right;
		ar << ratio_bottom;
		Serialize_Font(ar, &lfWeekday);
		ar << theApp.csWkdayFont;
		ar << theApp.WkdayColor;
		ar << theApp.nDt_Swap;
		Serialize_Font(ar, &lfNotes);
		ar << theApp.csNotesFont;
		ar << theApp.NotesColor;
		ar << theApp.nNoteLarge;
		Serialize_Font(ar, &lfTitle);
		ar << theApp.csTitleFont;
		ar << theApp.TitleColor;
		ar << nTitleLarge;
		Serialize_Font(ar, &lfDate);
		ar << theApp.csDateFont;
		ar << theApp.DateColor;
		ar << theApp.nDateLarge;
		ar << theApp.nDateSwap;
		ar << theApp.nDt_Custom;
		ar << PrinterName;
		ar << Orientation;
		ar << PaperSize;
		ar << PrintColor;
		ar << fGraphic;
		// If we have an image, save it and its info.
		if(fGraphic)
		{
			m_dib.Serialize(ar);
			ar << csImageName;
			ar << gratio_left;
			ar << gratio_top;
			ar << gratio_right;
			ar << gratio_bottom;
			ar << theApp.gAspect;
		}
	}
	else
	{
		for (int x=0; x<7; x++)
			for (int y=0; y<6; y++)
				ar >> m_grid[x][y];
		// In case there is an image loaded, delete it.
		theApp.Main_Pointer()->m_ImageDlg.Delete_Image();

		ar >> EventFile;
		ar >> nYear;
		ar >> nMonth;
		ar >> fMonYr;
		ar >> fTransparent;
		ar >> cstitle;
		ar >> ratio_left;
		ar >> ratio_top;
		ar >> ratio_right;
		ar >> ratio_bottom;
		Serialize_Font(ar, &lfWeekday);
		ar >> theApp.csWkdayFont;
		ar >> theApp.WkdayColor;
		ar >> theApp.nDt_Swap;
		Serialize_Font(ar, &lfNotes);
		ar >> theApp.csNotesFont;
		ar >> theApp.NotesColor;
		ar >> theApp.nNoteLarge;
		Serialize_Font(ar, &lfTitle);
		ar >> theApp.csTitleFont;
		ar >> theApp.TitleColor;
		ar >> nTitleLarge;
		Serialize_Font(ar, &lfDate);
		ar >> theApp.csDateFont;
		ar >> theApp.DateColor;
		ar >> theApp.nDateLarge;
		ar >> theApp.nDateSwap;
		ar >> theApp.nDt_Custom;
		ar >> PrinterName;
		ar >> Orientation;
		ar >> PaperSize;
		ar >> PrintColor;
		ar >> fGraphic;
		// If we have an image, retrieve it and its info.
		if(fGraphic)
		{
			m_dib.Serialize(ar);
			ar >> csImageName;
			ar >> gratio_left;
			ar >> gratio_top;
			ar >> gratio_right;
			ar >> gratio_bottom;
			ar >> theApp.gAspect;
		}
		// Clear the new document flag.
		fNewDoc = 0;
		// If we have a Month calendar, select the saved Event File name if it exists.
		if(!fMonYr)
		{
			if(pMain->m_EventsDlg.Select_Event_File(EventFile))
				pMain->m_EventsDlg.m_InsertDate.EnableWindow(TRUE);
			// Fill the Event List.
			pMain->m_EventsDlg.Read_Events();
			// Set the Event Save, Delete, Load button text.
			pMain->m_EventsDlg.m_EventSave.SetWindowText("Delete");
			if(pMain->m_EventsDlg.m_EventList.GetCount() != 0)
				pMain->m_EventsDlg.m_EventSave.EnableWindow(TRUE);
			// Let the View know that we have new events.
			theApp.View_Pointer()->fEvents++;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseDoc diagnostics

#ifdef _DEBUG
void CDaysEaseDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDaysEaseDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDaysEaseDoc commands

BOOL CDaysEaseDoc::Add_Data(CString cData, int x, int y)
{
	// Make sure we have a valid Day Box and add the string to the array.
	if ((x >= 0) && (x < 7) &&
        (y >= 0) && (y < 6))
		(m_grid[x][y]) = cData;
	return TRUE;
}

CString CDaysEaseDoc::GetSquare(int x, int y)
{
	// Make sure we have a valid Day Box and return the string from the array.
	if ((x >= 0) && (x < 7) &&
        (y >= 0) && (y < 6))
		return(m_grid[x][y]);
    return "";
}

void CDaysEaseDoc::Serialize_Font(CArchive& ar, LOGFONT* lfFont)
{
	if(ar.IsStoring())
	{
		ar << lfFont->lfHeight;
		ar << lfFont->lfWidth;
		ar << lfFont->lfEscapement;
		ar << lfFont->lfOrientation;
		ar << lfFont->lfWeight;
		ar << lfFont->lfItalic;
		ar << lfFont->lfUnderline;
		ar << lfFont->lfStrikeOut;
		ar << lfFont->lfCharSet;
		ar << lfFont->lfOutPrecision;
		ar << lfFont->lfClipPrecision;
		ar << lfFont->lfQuality;
		ar << lfFont->lfPitchAndFamily;
	}
	else
	{
		ar >> lfFont->lfHeight;
		ar >> lfFont->lfWidth;
		ar >> lfFont->lfEscapement;
		ar >> lfFont->lfOrientation;
		ar >> lfFont->lfWeight;
		ar >> lfFont->lfItalic;
		ar >> lfFont->lfUnderline;
		ar >> lfFont->lfStrikeOut;
		ar >> lfFont->lfCharSet;
		ar >> lfFont->lfOutPrecision;
		ar >> lfFont->lfClipPrecision;
		ar >> lfFont->lfQuality;
		ar >> lfFont->lfPitchAndFamily;
	}
}
