// GraphTracker.cpp: implementation of the CGraphTracker class.
//
/*
	DaysEase - PC Magazine file download utility
	Copyright  1998-1999 Ziff-Davis, Inc.  All rights reserved.
	First published in PC Magazine, US Edition
	December 1, 1998.
	Written by Richard A. Ellingson
*/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DaysEase.h"
#include "GraphTracker.h"
#include "DaysEaseView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
extern CDaysEaseApp theApp;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGraphTracker::CGraphTracker()
{
	m_nStyle = CRectTracker::resizeOutside | CRectTracker::dottedLine;
	m_nHandleSize = 10;
	m_sizeMin.cx = 30;
	m_sizeMin.cy = 30;
	fShift = 0;
}

CGraphTracker::~CGraphTracker()
{

}
void CGraphTracker::AdjustRect(int nHandle, LPRECT lpRect)
{
	CDaysEaseView* pView = theApp.View_Pointer();
	CRect rect;
	POINT point;
	double Aspect = theApp.gAspect;

	// Initialize the width.
	int nWidth = m_rect.right - m_rect.left;
	// If the width is smaller than minimum, make it the minimum.
	if(nWidth < m_sizeMin.cx)
		nWidth = m_sizeMin.cx;
	// Initialize the height.
	int nHeight = m_rect.bottom - m_rect.top;
	// If the height is smaller than minimum, make it the minimum.
	if(nHeight < m_sizeMin.cy)
		nHeight = m_sizeMin.cy;
	// Set a center point for width.
	int x = m_rect.left + nWidth/2;
	// Set a center point for height.
	int y = m_rect.top + nHeight/2;
	// Derive a ratio which would be half the height if
	// the aspect ratio is restored using the current width.
	int nWidth_ratio = (int)(((double)nWidth/Aspect)/2);
	// Derive a ratio which would be half the width if
	// the aspect ratio is restored using the current height.
	int nHeight_ratio = (int)(((double)nHeight*Aspect)/2);
	// Confine the rectangle.
	Clip_To();
	// Create a reference point to represent the top left of the page rectangle.
	point = theApp.page_rect.TopLeft();
	// Map the reference point to the screen coordinates.
	ClientToScreen(pView->m_hWnd, &point);
	// Create a screen coordinate rectangle equivalent to the page rectangle.
	rect.SetRect(point.x, point.y, point.x + theApp.page_rect.Width(), point.y + theApp.page_rect.Height());
	// Make it a little bigger.
	rect.InflateRect(10, 10);
	// Confine the mouse pointer to the new rectangle.
	ClipCursor(rect);
	// Perform the appropriate action depending on which resize handle is captured.
	switch(nHandle)
	{
	// TopLeft handle.
	case 0:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		if(fShift)
		{
			// Set the top equal to the center of the height minus half the calculated aspect height.
			m_rect.top = y - nWidth_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded top margin, then complete the aspect restoration using the left.
			if(m_rect.top <= theApp.page_rect.top)
				m_rect.left = m_rect.right -
					(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			else
			// If we did not exceeded top margin, then try to complete the aspect restoration using the left.
			{
				// Set the left equal to the center of the width minus half the calculated aspect width.
				m_rect.left = x - nHeight_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded left margin, then complete the aspect restoration using the top.
				if(m_rect.left <= theApp.page_rect.left)
					m_rect.top = m_rect.bottom - 
						(int)((double)(m_rect.right - m_rect.left)/Aspect);
			}
		}
		break;
	// TopRighthandle.
	case 1:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		if(fShift)
		{
			// Set the top equal to the center of the height minus half the calculated aspect height.
			m_rect.top = y - nWidth_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded top margin, then complete the aspect restoration using the right.
			if(m_rect.top == theApp.page_rect.top)
				m_rect.right = m_rect.left +
					(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			else
			// If we did not exceeded top margin, then try to complete the aspect restoration using the right.
			{
				// Set the right equal to the center of the width minus half the calculated aspect width.
				m_rect.right = x + nHeight_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded right margin, then complete the aspect restoration using the top.
				if(m_rect.right == theApp.page_rect.right)
					m_rect.top = m_rect.bottom - 
						(int)((double)(m_rect.right - m_rect.left)/Aspect);
			}
		}
		break;
	// BottomRighthandle.
	case 2:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		if(fShift)
		{
			// Set the right equal to the center of the width plus half the calculated aspect width.
			m_rect.right = x + nHeight_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded right margin, then complete the aspect restoration using the bottom.
			if(m_rect.right == theApp.page_rect.right)
				m_rect.bottom = m_rect.top +
					(int)((double)(m_rect.right - m_rect.left)/Aspect);
			else
			// If we did not exceeded right margin, then try to complete the aspect restoration using the bottom.
			{
				// Set the bottom equal to the center of the height plus half the calculated aspect height.
				m_rect.bottom = y + nWidth_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded bottom margin, then complete the aspect restoration using the right.
				if(m_rect.bottom == theApp.page_rect.bottom)
					m_rect.right = m_rect.left +
						(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			}
		}
		break;
	// BottomLefthandle.
	case 3:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		if(fShift)
		{
			// Set the left equal to the center of the width plus half the calculated aspect width.
			m_rect.left = x - nHeight_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded left margin, then complete the aspect restoration using the bottom.
			if(m_rect.left == theApp.page_rect.left)
				m_rect.bottom = m_rect.top +
					(int)((double)(m_rect.right - m_rect.left)/Aspect);
			else
			// If we did not exceeded left margin, then try to complete the aspect restoration using the bottom.
			{
				// Set the bottom equal to the center of the height plus half the calculated aspect height.
				m_rect.bottom = y + nWidth_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded bottom margin, then complete the aspect restoration using the left.
				if(m_rect.bottom == theApp.page_rect.bottom)
					m_rect.left = m_rect.right -
						(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			}
		}
		break;
	// Top handle.
	case 4:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		// Skip if the height is minimized.
		if(fShift && (nHeight > m_sizeMin.cy))
		{
			// Set the left equal to the center of the width minus half the calculated aspect width.
			m_rect.left = x - nHeight_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded left margin, then complete the aspect restoration using the right.
			if(m_rect.left == theApp.page_rect.left)
				m_rect.right = m_rect.left +
					(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			else
			// If we did not exceeded left margin, then try to complete the aspect restoration using the right.
			{
				// Set the right equal to the center of the width plus half the calculated aspect width.
				m_rect.right = x + nHeight_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded right margin, then complete the aspect restoration using the left.
				if(m_rect.right == theApp.page_rect.right)
					m_rect.left = m_rect.right -
						(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			}
		}
		break;
	// Right handle.
	case 5:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		// Skip if the width is minimized.
		if(fShift && (nWidth > m_sizeMin.cx))
		{
			// Set the top equal to the center of the height minus half the calculated aspect height.
			m_rect.top = y - nWidth_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded top margin, then complete the aspect restoration using the bottom.
			if(m_rect.top == theApp.page_rect.top)
				m_rect.bottom = m_rect.top +
					(int)((double)(m_rect.right - m_rect.left)/Aspect);
			else
			// If we did not exceeded top margin, then try to complete the aspect restoration using the bottom.
			{
				// Set the bottom equal to the center of the height plus half the calculated aspect height.
				m_rect.bottom = y + nWidth_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded bottom margin, then complete the aspect restoration using the top.
				if(m_rect.bottom == theApp.page_rect.bottom)
					m_rect.top = m_rect.bottom - 
						(int)((double)(m_rect.right - m_rect.left)/Aspect);
			}
		}
		break;
	// Bottomhandle.
	case 6:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		// Skip if the height is minimized.
		if(fShift && (nHeight > m_sizeMin.cy))
		{
			// Set the left equal to the center of the width minus half the calculated aspect width.
			m_rect.left = x - nHeight_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded left margin, then complete the aspect restoration using the right.
			if(m_rect.left == theApp.page_rect.left)
				m_rect.right = m_rect.left +
					(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			else
			// If we did not exceeded left margin, then try to complete the aspect restoration using the right.
			{
				// Set the right equal to the center of the width plus half the calculated aspect width.
				m_rect.right = x + nHeight_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded right margin, then complete the aspect restoration using the left.
				if(m_rect.right == theApp.page_rect.right)
					m_rect.left = m_rect.right -
						(int)((double)(m_rect.bottom - m_rect.top)*Aspect);
			}
		}
		break;
	// Lefthandle.
	case 7:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		// Skip if the width is minimized.
		if(fShift && (nWidth > m_sizeMin.cx))
		{
			// Set the top equal to the center of the height minus half the calculated aspect height.
			m_rect.top = y - nWidth_ratio;
			// Confine the rectangle.
			Clip_To();
			// If we exceeded top margin, then complete the aspect restoration using the bottom.
			if(m_rect.top == theApp.page_rect.top)
				m_rect.bottom = m_rect.top +
					(int)((double)(m_rect.right - m_rect.left)/Aspect);
			else
			// If we did not exceeded top margin, then try to complete the aspect restoration using the bottom.
			{
				// Set the bottom equal to the center of the height plus half the calculated aspect height.
				m_rect.bottom = y + nWidth_ratio;
				// Confine the rectangle.
				Clip_To();
				// If we exceeded bottom margin, then complete the aspect restoration using the top.
				if(m_rect.bottom == theApp.page_rect.bottom)
					m_rect.top = m_rect.bottom - 
						(int)((double)(m_rect.right - m_rect.left)/Aspect);
			}
		}
		break;
	// Middleor "move" handle.
	case 8:
		// If the Shift key is depressed, we must maintain current aspect ratio.
		if(fShift)
		{
			// Since the width and height are not allowed to change,
			// restore any breach using the hold rectangle height and width.
			if(m_rect.right >= theApp.page_rect.right)
				m_rect.left = theApp.page_rect.right - hold_rect.Width(); 
			if(m_rect.left <= theApp.page_rect.left)
				m_rect.right = theApp.page_rect.left + hold_rect.Width();
			if(m_rect.bottom >= theApp.page_rect.bottom)
				m_rect.top = theApp.page_rect.bottom - hold_rect.Height();
			if(m_rect.top <= theApp.page_rect.top)
				m_rect.bottom = theApp.page_rect.top + hold_rect.Height(); 
		}
	}
	// Confine the rectangle.
	Clip_To();
	// Important: always be sure to release the mouse pointer.
	ClipCursor(NULL);
	CRectTracker::AdjustRect(nHandle, lpRect);
}

void CGraphTracker::Clip_To()
{
	// If any margin is breached, confine to the margin and restore
	// the minimum size by adjusting the opposite side.
	if(m_rect.right > theApp.page_rect.right)
	{
		m_rect.right = theApp.page_rect.right;
		if((m_rect.right - m_rect.left) < m_sizeMin.cx)
			m_rect.left = m_rect.right - m_sizeMin.cx;
	}
	if(m_rect.left < theApp.page_rect.left)
	{
		m_rect.left = theApp.page_rect.left;
		if((m_rect.right - m_rect.left) < m_sizeMin.cx)
			m_rect.right = m_rect.left + m_sizeMin.cx;
	}
	if(m_rect.bottom > theApp.page_rect.bottom)
	{
		m_rect.bottom = theApp.page_rect.bottom;
		if((m_rect.bottom - m_rect.top) < m_sizeMin.cy)
			m_rect.top = m_rect.bottom - m_sizeMin.cy;
	}
	if(m_rect.top < theApp.page_rect.top)
	{
		m_rect.top = theApp.page_rect.top;
		if((m_rect.bottom - m_rect.top) < m_sizeMin.cy)
			m_rect.bottom = m_rect.top + m_sizeMin.cy;
	}
}
